"""Reproject and resample Cordex data to target grid."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import re
import sys
import logging
from joblib import Parallel, delayed
from logging.config import dictConfig

# locals
from pysegcnn.core.logging import log_conf
from climax.core.utils import get_inventory, reproject_cdo
from climax.core.cli import preprocess_parser
from climax.core.constants import EUROCORDEX_DOMAIN

# module level logger
LOGGER = logging.getLogger(__name__)


def _check_str_arg(arg, pattern):
    # check if input argument matches a defined pattern
    if arg is not None:
        arg_id = re.search(pattern, arg)
        if arg_id is None:
            LOGGER.info('Argument "{}" invalid, form "{}" required.'
                        .format(arg, pattern))
            sys.exit()
        else:
            arg = arg_id[0]

    return arg


if __name__ == '__main__':

    # configure logging
    dictConfig(log_conf())

    # define command line argument parser
    parser = preprocess_parser()

    # parse command line arguments
    args = sys.argv[1:]
    if not args:
        parser.print_help()
        sys.exit()
    else:
        args = parser.parse_args(args)

    # check whether the source directory exists
    if args.source.exists():
        # check whether the target grid file exists
        if not args.grid.exists():
            LOGGER.info('{} does not exist.'.format(args.grid))
            sys.exit()

        # create target directory: check that it is differen from the source
        # directory
        if args.target == args.source:
            LOGGER.info('Source and target paths cannot be equal.')
            sys.exit()

        if not args.target.exists():
            LOGGER.info('mkdir {}'.format(args.target))
            args.target.mkdir(parents=True, exist_ok=True)

        # check ensemble identifier
        ensemble = _check_str_arg(args.ensemble, 'r[0-9]i[0-9]p[0-9]')

        # check regional climate model version
        version = _check_str_arg(args.version, 'v[0-9]')

        # construct file pattern to match from input parameters
        pattern = '(.*).nc$'
        pattern = '_'.join([param if param is not None else '(.*)' for param in
                            [args.variable, EUROCORDEX_DOMAIN, args.gcm,
                             args.scenario, ensemble, args.rcm, version,
                             pattern]])

        # get all the files matching the defined pattern in the source
        # directory
        source = get_inventory(args.source, pattern, return_df=False)

        # extract files of the specified variable
        LOGGER.info('Files matching "{}" for variable "{}":'.format(
            pattern, args.variable))
        LOGGER.info(('\n ' + (len(__name__) + 1) * ' ').join(
                    ['{}'.format(file) for file in source]))

        # check whether to only print which files would be processed
        if not args.dry_run:
            # generate target filenames
            target = [args.target.joinpath(file.name) for file in source]

            # run reprojection in parallel
            target = Parallel(n_jobs=-1, verbose=51)(
                delayed(reproject_cdo)(args.grid, src, trg, args.mode,
                                       args.overwrite)
                for src, trg in zip(source, target))

    else:
        LOGGER.info('{} does not exist.'.format(str(args.source)))
        sys.exit()
